<?php
// $Id: ec_file.inc,v 1.1.2.3 2008/01/13 23:10:21 brmassa Exp $
/**
 * @author e-Commerce Dev Team
 * @file ec_file.module
 * Create file products. Enable the download only after the
 * payment and set date to expire.
 */


/**
 * Implementation of hook_file_download().
 */
function _ec_file_file_download($file) {
  global $user;
  static $may_download = array();
  $file = file_create_path($file);

  // Check if its not cached already
  if (!isset($may_download[$file][$user->uid])) {
    $data = db_fetch_object(db_query("SELECT st.txnid, stp.vid, f.filemime
      FROM {ec_transaction} AS st
      INNER JOIN {ec_transaction_product} AS stp ON stp.txnid = st.txnid
      INNER JOIN {ec_file_new} epf ON epf.nid = stp.nid
      INNER JOIN {files} f ON f.fid = epf.fid
      WHERE st.ecid = %d AND st.workflow = %d AND f.filepath = '%s'",
      $user->uid, EC_WORKFLOW_COMPLETED, $file));

    $may_download[$file][$user->uid] = FALSE;

    if (!empty($data)) {
      if (!ec_recurring_product_get_expiry($data->txnid, $data->vid)) {
        $may_download[$file][$user->uid] = $data->filemime;
      }
    }
  }

  if (!empty($may_download[$file][$user->uid])) {
    return array('Content-type: '. $may_download[$file][$user->uid]);
  }

  // Not allowed
  return -1;
}

/**
 * Add a new field on each Attachment. This will mark the file
 * as a product
 *
 * @param &$form
 *   Array, the form farray
 * @param $files
 *   Array, the form fields that deals with file attachments
 */
function _ec_file_form_change(&$form, $files) {
  foreach (element_children($files) as $id) {
    $files[$id]['product'] = array(
      '#default_value'  => db_result(db_query('SELECT fid FROM {ec_file_new}
          WHERE nid = %d and fid = %d', $form['nid']['#value'], $id)),
      '#type'           => 'checkbox'
    );
  }

  // Overwrite the theme function upload_form_current
  $files['#theme'] = 'ec_file_node_form';

  if ($form_id == 'upload_js') {
    $form['files'] = $files;
  }
  else {
    $form['attachments']['wrapper']['files'] = $files;
  }

  // Add a new submit function to save the products checkbox
  $form['#submit']['_ec_file_form_change_submit'] = array();
}

/**
 * Add a new field on each Attachment. This will mark the file
 * as a product
 *
 * @param &$files
 *   Array, the form fields
 * @param $vid
 *   Number, the ID
 */
function _ec_file_form_change_submit($form_id, &$form) {
  if (!empty($form['files'])) {
    db_query('DELETE FROM {ec_file_new} WHERE nid = %d', $form['nid']);
    foreach ($form['files'] as $files) {
      if (!empty($files['product'])) {
        db_query('INSERT INTO {ec_file_new} (nid, fid) VALUES (%d, %d)',
          $form['nid'], $files['fid']);
      }
    }
  }
}

/**
 * Print the files for the user to download.
 *
 * Note:
 * Expiration date is found via the sitewide file expiration date OR the
 * special case where a store admin has extended the expiration date for a given
 * transaction. The 'special case' always take precedence, so you can disable
 * downloads before the sitewide expiration date is reached.
 */
function __ec_file_my_overview($uid = FALSE) {
  $name = db_result(db_query('SELECT name FROM {users} WHERE uid = %d', $uid));
  $output = '';

  for ($expired = 0; $expired <= 1; $expired++) {
    if (empty($expired)) {
      $status = ECRECURRING_STATUS_ACTIVE;
    }
    else {
      $status = ECRECURRING_STATUS_EXPIRED;
    }

    $files[$expired] = array();
    $header = array(
      array('data' => t('filename'), 'field' => 't.txnid', 'sort' => 'desc'),
      array('data' => t('size'), 'field' => 'epf.size'),
      array('data' => ($expired) ? t('expired') : t('expires'), 'field' => 'ere.expiry'),
      t('operations')
    );

    // Check 1: Grab all explicit file downloads.
    $result = db_query('SELECT etp.title, f.filesize, f.filepath, t.workflow, ere.expiry
      FROM {ec_transaction} t
      INNER JOIN {ec_transaction_product} etp ON etp.txnid = t.txnid
      INNER JOIN {ec_file_new} epf ON epf.nid = etp.nid
      INNER JOIN {files} f ON f.fid = epf.fid
      LEFT OUTER JOIN {ec_recurring_expiration} ere ON ere.vid = etp.vid
      AND ere.txnid = etp.txnid
      WHERE t.ecid = %d AND ((ere.status = 0 AND ere.rid <= 0) OR ere.status IS NULL)'.
      tablesort_sql($header), $uid, $status);

    while ($data = db_fetch_object($result)) {
      if (!empty($cache[$data->filepath])) {
        continue;
      }
      $cache[$data->filepath] = TRUE;
      $files[$expired][] = $data;
    }
  }

  return theme('ec_files_myfiles', $files);
}

/**
 * List all files purchased and for download
 *
 * @param $files
 *   Array, the list of files
 * @return
 *   String, the HMTL with all files purchased
 * @ingroup theme
 */
function theme_ec_files_myfiles($files_list) {
  $header = array(
    array('data' => t('filename'), 'field' => 't.txnid', 'sort' => 'desc'),
    array('data' => t('size'), 'field' => 'epf.size'),
    array('data' => ($expired) ? t('expired') : t('expires'), 'field' => 'ere.expiry'),
    array('data' => t('Transaction'))
  );

  foreach ($files_list as $expired => $files) {
    $rows = array();
    foreach ($files as $file) {
      // Set the file name
      $file->title .= ' ('. preg_replace('|^[^/]*/|', '', $file->filepath) .')';
      if ($file->workflow == EC_WORKFLOW_COMPLETED and empty($expired)) {
        $file->title = l($file->title, 'system/'. $file->filepath);
      }
      else {
        $file->title = theme('placeholder', $file->title);
      }

      // Create the expiration data string
      if (empty($file->expiry)) {
        $file->expiry = '-';
      }
      else {
        if (!empty($expired)) {
          $file->expiry = t('%interval ago',
            array('%interval' => format_interval($file->expiry - time(), 3)));
        }
        else {
          $file->expiry = format_interval($file->expiry - time(), 3);
        }
      }

      $rows[] = array($file->title, format_size($file->filesize), $file->expiry,
        transaction_get_workflow($file->workflow));
    }

    if (empty($files)) {
      if (empty($expired)) {
        $output .= t('You have no files to download.');
      }
    }
    else {
      if (!empty($expired)) {
        $output .= '<h3>'. t('Expired files') .'</h3>';
      }
      $output .= theme('table', $header, $rows);
    }
  }
  return $output;
}
