<?php
// $Id: shipping.inc,v 1.1.2.1 2008/01/11 10:29:18 brmassa Exp $
/**
 * @file shipping.inc
 * All non-hook function
 */


/**
 * Filter a set of shipping methods.
 *
 * Arrays of methods are generally in one of two different formats.
 * The first is that returned by the _shipping_methods hook, e.g,
 *
 * <code>
 * $methods['ups'] = array(
 *    '#title' => t('UPS'),
 *    '#description' => t('United Parcel Service of America, Inc.')
 *  );
 *  $methods['ups']['1DM'] = array(
 *    '#title' => t('Next Day Air Early AM'),
 *  );
 * </code>
 *
 * The second is that used to pass product type and product shipping
 * options, e.g.,
 *
 * <code>
 * array('ups' => array('1DM', '1DA'));
 * </code>
 *
 * This function is used to filter method arrays of the first type (the
 * first argument), returning only items with corresponding values in the
 * method of the second type (the second argument).
 */
function _shipping_methods_filter($methods, $used_methods) {

  foreach ($methods as $service => $method) {
    // First filter out base methods, e.g, 'ups'.
    if (!in_array($service, array_keys($used_methods)) || empty($used_methods[$service])) {
      unset($methods[$service]);
    }
    // For applicable methods, filter out any sub-methods not used, e.g., for 'ups', 'firstday'.
    else {
      foreach (element_children($method) as $key) {
        if (!in_array($key, $used_methods[$service])) {
          unset($methods[$service][$key]);
        }
      }
    }
  }

  return $methods;
}

