<?php
// $Id: store.admin.inc,v 1.1.2.65 2008/01/17 04:45:03 sime Exp $

/**
 * @file store.admin.inc
 * All store admin functions.
 */

/**
 * Implementation of hook_cron().
 */
function _store_cron() {
  // Any item whose workflow is 'shipped', move it to 'completed'. This does
  // seem presumptous. Ideally we'd email the user a list of shipped items for
  // day and then adjust the shipping workflow after a given time period, such
  // as 2 week (in case of returns or something) rather than automatically
  // closing it here.
  db_query('UPDATE {ec_transaction} SET workflow = %d
    WHERE workflow = %d AND allocation = %d', EC_WORKFLOW_COMPLETED,
    EC_WORKFLOW_SHIPPED, EC_ALLOC_COMPLETED);

  if (variable_get('payment_notices', 1) and
      (time() >= (variable_get('ecommerce_store_summary_email_sent', 0) +
      variable_get('ecommerce_store_summary_email_frequency', 60 * 60 * 24)))) {
    global $base_url;

    // Find order(s) with a 'received' payment status needs action.
    if ($count = db_result(db_query('SELECT COUNT(st.txnid)
      FROM {ec_transaction} AS st WHERE st.allocation = %d AND
      (st.workflow = %d OR st.workflow = %d OR st.workflow = %d OR st.workflow = %d)',
      EC_ALLOC_COMPLETED, EC_WORKFLOW_RECEIVED, EC_WORKFLOW_INVOICED,
      EC_WORKFLOW_SHIPPED, EC_WORKFLOW_WAITING_RESPONSE))) {

      $beg_month    = mktime(0, 0, 0, date('m'), 1, date('Y'));
      $mon_orders  = db_result(db_query('SELECT COUNT(txnid)
        FROM {ec_transaction} where created >= %d AND allocation = %d AND
        workflow = %d', $beg_month, EC_ALLOC_COMPLETED, EC_WORKFLOW_COMPLETED));
      $mon_sales    = '$'. number_format(db_result(db_query('SELECT SUM(gross)
        FROM {ec_transaction} where created >= %d AND allocation = %d AND
        workflow = %d', $beg_month, EC_ALLOC_COMPLETED, EC_WORKFLOW_COMPLETED)), 2);

      $buf = t('@count order(s) with a "cleared" payment status needs action:',
        array('@count' => $count)) ."\n";
      $result = db_query('SELECT * FROM {ec_transaction} AS st
        WHERE st.allocation = %d AND st.workflow IN (%d, %d, %d, %d)
        ORDER BY st.txnid DESC', EC_ALLOC_COMPLETED, EC_WORKFLOW_RECEIVED,
        EC_WORKFLOW_INVOICED, EC_WORKFLOW_SHIPPED, EC_WORKFLOW_WAITING_RESPONSE);
      while ($data = db_fetch_object($result)) {

        $buf .= t('Order #@txnid created @timestamp ago', array('@txnid' => $data->txnid,
          '@timestamp' => format_interval(time() - $data->created, 2))) ."\n".
          t('Workflow status: @transaction-workflow',
          array('@transaction-workflow' => transaction_get_workflow($data->workflow))) ."\n".
          t('Email: @email', array('@email' => $data->mail)) ."\n".
          t('Total: @gross', array('@gross' => format_currency($data->gross))) ."\n";

        $r = db_query('SELECT stp.*, st.allocation, st.workflow
          FROM {ec_transaction} AS st, {ec_transaction_product} AS stp
          WHERE st.txnid = %d AND st.txnid = stp.txnid
          ORDER BY st.txnid DESC', $data->txnid);
        $items = null;
        $has_shippable_item = false;
        while ($item = db_fetch_object($r)) {
          if ($product = product_load($item)) {
            foreach ($product as $key => $value) {
              if ($key != 'title' && $key != 'price') {
                $item->$key = $value;
              }
            }
          }
          if (product_is_shippable($item->vid)) {
            $has_shippable_item = true;
          }

          $sku = ($item->sku) ? " [$item->sku]" : "";
          $items .= t('@order of @title at @price each', array(
            '@order' => format_plural($item->qty, '1 order', '@count orders'),
            '@title' => $item->title . $sku,
            '@price' => format_currency($item->price)));
        }

        // If the transaction has no shippable items and the payment status is
        // 'complete' and the workflow is 'transaction received', set the
        // workflow to 'complete'
        if (empty($has_shippable_item) && $data->allocation == EC_ALLOC_COMPLETED && $data->workflow == EC_WORKFLOW_RECEIVED) {
          db_query('UPDATE {ec_transaction} SET workflow = %d WHERE txnid = %d', EC_WORKFLOW_COMPLETED, $data->txnid);
        }

        include_once drupal_get_path('module', 'store') .'/store.inc';
        $shipping_to = theme('store_format_address', $data, 'shipping', 'text');
        $billing_to = theme('store_format_address', $data, 'billing', 'text');

        $buf .= t("Shipping to:\n!shipping_address", array('!shipping_address' => $shipping_to)) ."\n".
          t("Billing to:\n!billing_address", array('!billing_address' => $billing_to)) ."\n".
          "$items\n".
          t('Update:') ."$base_url/admin/store/transaction/edit/$data->txnid\n".
          "=====\n";

      }

      $output = $buf;
      $output .= t('(These orders will no longer be listed once the transaction workflow is set to either "Completed" or "Canceled")') .
        "\n". t("Monthly Summary\nNumber of completed orders: @monthly-orders\nSales: @monthly-sales",
        array('@monthly-orders' => $mon_orders, '@monthly-sales' => $mon_sales)) ."\n".
        t('Want more reports? Visit @base_url/admin/store', array('@base_url' => $base_url)) ."\n";

      $subject    = t('@site-name transaction summary', array('@site-name' => variable_get('site_name', 'Drupal')));
      $to         = variable_get('site_mail', ini_get('sendmail_from'));
      $from       = variable_get('site_mail', ini_get('sendmail_from'));
      $headers    = array(
        'From'        => $from,
        'Reply-to'    => $from,
        'X-Mailer'    => 'PHP',
        'Return-path' => $from,
        'Errors-to'   => $from,
      );

      drupal_mail('ec_order_summary', $to, $subject, $output, $from, $headers);
      variable_set('ecommerce_store_summary_email_sent', time());
    }
  } // End receive payment notices.
}

/**
 * Called by store_ec_settings
 */
function _store_ec_settings() {
  $site_mail = variable_get('site_mail', ini_get('sendmail_from'));
  if (empty($site_mail)) {
    $site_mail = 'no e-mail address found';
  }

  $form['ec_order_overview'] = array(
    '#type'           => 'textarea',
    '#title'          => t('Order history overview'),
    '#default_value'  => variable_get('ec_order_overview', ''),
    '#cols'           => 70,
    '#rows'           => 5,
    '#description'    => t('This text will be displayed at the top of users\' order history pages.'),
  );

  $form['product_cart_addition_by_link'] = array(
    '#type'           => 'checkbox',
    '#title'          => t('Users adds product to cart using link.'),
    '#default_value'  => variable_get('product_cart_addition_by_link', 1),
    '#description'    => t('If this field is unchecked a form will be placed at the bottom of the product view which will allow the user select the quantity and other information.'),
  );

  // Links settings
  $form['ec_links'] = array(
    '#type'           => 'fieldset',
    '#title'          => t('Links'),
    '#collapsible'    => true,
    '#collapsed'      => true
  );
  $form['ec_links']['ec_goto_cart_change'] = array(
    '#type'           => 'textfield',
    '#title'          => t('After add a item link'),
    '#default_value'  => variable_get('ec_goto_cart_change', 'cart/view'),
    '#description'    => t('Enter the user destination after adding a product to the cart (eg. "node/1"). If left blank, the user will remain on the same page and a message will alert them that the item has been added to their cart. To send the user to cart, enter "cart/view".'),
  );
  $form['ec_links']['ec_goto_cart_empty'] = array(
    '#type'           => 'textfield',
    '#title'          => t('Empty cart link'),
    '#default_value'  => variable_get('ec_goto_cart_empty', ''),
    '#description'    => t('If the user cart is empty, you can send him/her to a given page.'),
  );

  // Transaction Notices settings
  $form['ec_notices'] = array(
    '#type'           => 'fieldset',
    '#title'          => t('Transaction notices'),
    '#collapsible'    => true,
    '#collapsed'      => true,
    '#description'    => t('These settings control whether the site administrator will receive email notifications of transaction reports, and the frequency thereof.'),
  );
  $form['ec_notices']['payment_notices'] = array(
    '#type'           => 'radios',
    '#title'          => t('Transaction e-mail notices'),
    '#default_value'  => variable_get('payment_notices', 0),
    '#options'        => array(t('Disabled'), t('Enabled')),
    '#description'    => t('Enable or disable transaction notifications. If enabled, the site administrator (%site_mail) will receive an email summarizing the status of all orders. The frequency of emails is dependent on how often the site administrator has scheduled to run cron jobs.  Finally, The site administrator e-mail address can be changed in the <a href="!general_config_screen">general configuration screen</a>,',
      array('%site_mail' => $site_mail,
      '!general_config_screen' => url('admin/settings')))
  );
  $form['ec_notices']['ecommerce_store_summary_email_frequency'] = array(
    '#type'           => 'select',
    '#title'          => t('Frequency of notices'),
    '#options'        => drupal_map_assoc(
      array(0, 60*60, 60*60*12, 60*60*24, 60*60*24*5, 60*60*24*7),
      'format_interval'),
    '#default_value'  => variable_get('ecommerce_store_summary_email_frequency', 60*60*24),
    '#description'    => t('As long as cron is running, this ensures a transaction report is sent no more than once in this period. Thus, even if cron runs every hour, you will still only receive once transaction notification e-mail.'),
  );

  // Email notification settings
  $form['email_notification'] = array(
    '#type'           => 'fieldset',
    '#title'          => t('Email Notifications'),
    '#collapsible'    => true,
    '#collapsed'      => true
  );
  $form['email_notification']['customer_invoice'] = ec_mail_selection_form_for_variable('store_settings_form',
    ECMAIL_TYPE_CUSTOMER_INVOICE, t('Customer invoice e-mail'), MAILVAR_CUSTOMER_INVOICE,
    t('This text will be emailed to customers after they make a purchase.'));
  $form['email_notification']['processing_error'] = ec_mail_selection_form_for_variable('store_settings_form',
    ECMAIL_TYPE_PROCESSING_ERROR, t('Payment error e-mail'), MAILVAR_PROCESSING_ERROR,
    t('This text will be e-mailed to customers if there is a payment processing problem.'));
  $form['email_notification']['ask_customer_template'] = ec_mail_selection_form_for_variable('store_settings_form',
    ECMAIL_TYPE_ASK_CUSTOMER, t('Ask customer e-mail template'), MAILVAR_ASK_CUSTOMER,
    t('This text is used as a template for asking customer.'));
  $form['email_notification']['cancel_transaction'] = ec_mail_selection_form_for_variable('store_settings_form',
    ECMAIL_TYPE_CANCEL_TXN, t('Cancel transaction e-mail'), MAILVAR_CANCEL_TXN,
    t('This text will be e-mailed to customers if their purchase has been canceled.'));

  return system_settings_form($form);
}

/**
 * Set all Location settings, like the used physical
 * dimenstions and currency format
 *
 * @return
 *   Array. The form settings array.
 */
function _store_location_settings_form() {
  $countries = _store_location_countries();
  $form['ec_country'] = array(
    '#type' => 'select',
    '#title' => t('Store location'),
    '#default_value' => variable_get('ec_country', 'us'),
    '#options' => $countries,
    '#multiple' => FALSE,
    '#description' => 'Default country location.',
  );

  // Physical dimensions
  $measures = _store_location_measures();
  $form['dimensions'] = array(
    '#collapsible'    => TRUE,
    '#title'          => t('Physical Dimensions'),
    '#tree'           => FALSE,
    '#type'           => 'fieldset'
  );
  $form['dimensions']['ec_measure_weight'] = array(
    '#default_value'  => variable_get('ec_measure_weight', 'LB'),
    '#description'    => t('Default unit measure of weight.'),
    '#options'        => $measures['weight'],
    '#title'          => t('Weight'),
    '#type'           => 'select'
  );
  $form['dimensions']['ec_measure_length'] = array(
    '#default_value'  => variable_get('ec_measure_length', 'IN'),
    '#description'    => t('Default unit measure of length.'),
    '#options'        => $measures['length'],
    '#title'          => t('Length'),
    '#type'           => 'select'
  );
  $form['dimensions']['ec_measure_area'] = array(
    '#default_value'  => variable_get('ec_measure_area', 'IN2'),
    '#description'    => t('Default unit measure of area.'),
    '#options'        => $measures['area'],
    '#title'          => t('Area'),
    '#type'           => 'select'
  );
  $form['dimensions']['ec_measure_volume'] = array(
    '#default_value'  => variable_get('ec_measure_volume', 'IN3'),
    '#description'    => t('Default unit measure of volume.'),
    '#options'        =>  $measures['volume'],
    '#title'          => t('Volume'),
    '#type'           => 'select'
  );

  // Price format settings
  $form['price_format'] = array(
    '#collapsible'    => TRUE,
    '#title'          => t('Price Formatting'),
    '#tree'           => FALSE,
    '#type'           => 'fieldset',
  );
  $form['price_format']['payment_symbol'] = array(
    '#type'           => 'textfield',
    '#title'          => t('Currency Symbol'),
    '#default_value'  => variable_get('payment_symbol', '$'),
    '#size'           => 3,
    '#maxlength'      => 5,
    '#description'    => t('Enter the currency symbol you wish to associate with your price. This will be displayed in front of the price. Default is the dollar symbol.')
  );
  $form['price_format']['payment_symbol_position'] = array(
    '#type'           => 'radios',
    '#title'          => t('Currency Symbol'),
    '#default_value'  => variable_get('payment_symbol_position', 1),
    '#options'        => array(t('Right'), t('Left')),
    '#description'    => t('This option places the currency symbol of the left or right side of the price.')
  );
  $form['price_format']['payment_thousands'] = array(
    '#type'           => 'textfield',
    '#title'          => t('Thousands separator'),
    '#default_value'  => variable_get('payment_thousands', ','),
    '#size'           => 3,
    '#maxlength'      => 5,
    '#description'    => t('Enter the sign for the thousands separator.')
  );
  $form['price_format']['payment_decimal'] = array(
    '#type'           => 'textfield',
    '#title'          => t('Decimal separator'),
    '#default_value'  => variable_get('payment_decimal', '.'),
    '#size'           => 3,
    '#maxlength'      => 5,
    '#description'    => t('Enter the sign to seperate real numbers from floating numbers.')
  );
  $form['price_format']['payment_decimal_places'] = array(
    '#type'           => 'textfield',
    '#title'          => t('Number of places after the decimal separator'),
    '#default_value'  => variable_get('payment_decimal_places', 2),
    '#size'           => 3,
    '#maxlength'      => 5,
    '#description'    => t('How many slots are needed after the decimal?')
  );

  return system_settings_form($form);
}

/**
 * Called by store_transaction_form()
 */
function _store_transaction_form($edit = null) {
  include_once drupal_get_path('module', 'store') .'/store.inc';
  $edit = is_array($edit) ? (object) $edit : $edit;

  $section = $edit->section ? $edit->section : $_GET['section'];

  switch (strtoupper($section)) {
    case 'OVERVIEW':
      $form = store_transaction_overview_form($edit);
      break;

    case 'ADDRESSES':
      $form = store_transaction_addresses_form($edit);
      break;

    case 'ITEMS':
      $form = store_transaction_item_form($edit);
      break;

    default:
      $form = array_merge(store_transaction_overview_form($edit),
        store_transaction_addresses_form($edit), store_transaction_item_form($edit));
  }

  if ($edit->uid && is_numeric($edit->uid)) {
    $edit->uid = db_result(db_query('SELECT name FROM {users} WHERE uid = %d', $edit->uid));
  }

  if (!empty($edit->txnid)) {
    $form['txnid'] = array(
      '#type' => 'value',
      '#value' => $edit->txnid,
      );
    $form[] = array(
      '#type' => 'submit',
      '#value' => t('Update transaction')
      );
    $form[] = array(
      '#type' => 'submit',
      '#value' => t('Delete'),
      );
  }
  else {
    $form[] = array(
      '#type' => 'submit',
      '#value' => t('Create new transaction'),
      );
  }

  $form['section'] = array(
    '#type' => 'value',
    '#value' => $section,
    );

  return $form;
}

/**
 * Called by store_transactions_advanced_search_form()
 */
function _store_transactions_advanced_search_form($query) {
  $form['keywords1'] = array(
    '#prefix' => '<div class="criterion">',
    '#suffix' => '</div>'
  );
  $form['keywords1']['txnid'] = array(
    '#type' => 'textfield',
    '#title' => t('Transaction ID'),
    '#default_value' => empty($query['txnid']) ? '' : $query['txnid'],
    '#size' => 30,
    '#maxlength' => 255,
    '#description' => t('The ID number of a transaction.')
  );
  $form['keywords1']['nid'] = array(
    '#type' => 'textfield',
    '#title' => t('Product ID'),
    '#default_value' => empty($query['nid']) ? '' : $query['nid'],
    '#size' => 30,
    '#maxlength' => 255,
    '#description' => t('Find a transaction that has the following product(s).')
  );
  $form['keywords2'] = array('#prefix' => '<div class="criterion">', '#suffix' => '</div>');
  $form['keywords2']['nid_expires'] = array(
    '#type' => 'textfield',
    '#title' => t('Product expiration'),
    '#default_value' => empty($query['nid_expires']) ? '' : $query['nid_expires'],
    '#size' => 30,
    '#maxlength' => 255,
    '#description' => t('Date product expires.')
  );
  $form['keywords2']['date'] = array(
    '#type' => 'textfield',
    '#title' => t('Date created'),
    '#default_value' => empty($query['date']) ? '' : $query['date'],
    '#size' => 30,
    '#maxlength' => 255,
    '#description' => t('Date transaction was created.')
  );
  $form['keywords2']['duedate'] = array(
    '#type' => 'textfield',
    '#title' => t('Date due'),
    '#default_value' => empty($query['duedate']) ? '' : $query['duedate'],
    '#size' => 30,
    '#maxlength' => 255,
    '#description' => t('Date transaction was due.')
  );
  $form['keywords2']['gross'] = array(
    '#type' => 'textfield',
    '#title' => t('Total cost'),
    '#default_value' => empty($query['gross']) ? '' : $query['gross'],
    '#size' => 30,
    '#maxlength' => 255
  );
  $form['allocation'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Allocation status'),
    '#default_value' => empty($query['allocation']) ? '' : $query['allocation'],
    '#options' => store_transaction_allocation(),
    '#prefix' => '<div class="criterion">',
    '#suffix' => '</div>'
  );
  $form['workflow'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Workflow'),
    '#default_value' => empty($query['workflow']) ? '' : $query['workflow'],
    '#options' => store_transaction_workflow(),
    '#prefix' => '<div class="criterion">',
    '#suffix' => '</div>'
  );
  $form['ptype'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Product type'),
    '#default_value' => empty($query['ptype']) ? '' : $query['ptype'],
    '#options' => product_ptypes_get('names'),
    '#prefix' => '<div class="criterion">',
    '#suffix' => '</div>'
  );

  return $form;
}

/**
 * Overview Widget that shows some orders data.
 *
 * @param $widget
 *   Array (optional), The original widget options
 * @return
 *   Array, the widget data
 */
function _store_widget_orders($widget = array()) {
  $num_pending    = db_result(db_query('SELECT COUNT(txnid) FROM {ec_transaction}
    WHERE (allocation = %d OR workflow = %d) AND workflow IN (%d, %d, %d, %d)',
    EC_ALLOC_PENDING, EC_ALLOC_COMPLETED, EC_WORKFLOW_RECEIVED, EC_WORKFLOW_INVOICED,
    EC_WORKFLOW_SHIPPED, EC_WORKFLOW_WAITING_RESPONSE));
  $num_part_paid      = db_result(db_query('SELECT COUNT(txnid) FROM {ec_transaction} WHERE allocation = %d', EC_ALLOC_PART));
  $num_processing = db_result(db_query('SELECT COUNT(txnid) FROM {ec_transaction}
    WHERE allocation = %d AND workflow IN (%d, %d, %d, %d)', EC_ALLOC_COMPLETED,
    EC_WORKFLOW_RECEIVED, EC_WORKFLOW_INVOICED, EC_WORKFLOW_SHIPPED, EC_WORKFLOW_WAITING_RESPONSE));
  $num_completed  = db_result(db_query('SELECT COUNT(txnid) FROM {ec_transaction}
    WHERE allocation = %d and workflow = %d', EC_ALLOC_COMPLETED, EC_WORKFLOW_COMPLETED));
  $num_alloc_cancel = db_result(db_query('SELECT COUNT(txnid) FROM {ec_transaction} WHERE allocation IN (%d, %d) AND workflow = %d', EC_ALLOC_PART, EC_ALLOC_COMPLETED, EC_WORKFLOW_CANCELED));

  // Build the content table
  $content = theme('table', NULL, array(
    array(
      array('data' => l(t('Payments pending'), 'admin/store/transaction',
        array(), 'allocation=0,1&workflow=1,2,3,4'), 'align' => 'right',
        'title' => t('Payment status: Pending, workflow: Not Complete')),
      array('data' => $num_pending, 'align' => 'right')
    ),
    array(
      array('data' => l(t('Part Payments'), 'admin/store/transaction',
        array(), 'allocation=1'), 'align' => 'right',
        'title' => t('Payment status: Part')),
      array('data' => $num_part_paid, 'align' => 'right')
    ),
    array(
      array('data' => l(t('Orders needing action'), 'admin/store/transaction',
        array(), 'allocation=2&workflow=1,2,3,4'), 'align' => 'right',
        'title' => t('Payment status: Complete, workflow: Not Complete')),
      array('data' => $num_processing, 'align' => 'right')
    ),
    array(
      array('data' => l(t('Completed'), 'admin/store/transaction',
        array(), 'workflow=6'), 'align' => 'right'),
      array('data' => $num_completed, 'align' => 'right'),
    ),
    array(
      array('data' => l(t('Canceled with Allocated funds'), 'admin/store/transaction',
        array(), 'workflow=5&allocation=1,2'), 'align' => 'right'),
      array('data' => $num_alloc_cancel, 'align' => 'right'),
    )
  ));

  return $content;
}

/**
 * Overview Widget that shows some sales data.
 *
 * @param $widget
 *   Array (optional), The original widget options
 * @return
 *   Array, the widget data
 */
function _store_widget_sales($widget = array()) {
  $beg_month   = mktime(0, 0, 0, date('m'), 1, date('Y'));
  $beg_year    = mktime(0, 0, 0, 1, 1, date('Y'));
  $num_orders  = db_result(db_query('SELECT COUNT(txnid) FROM {ec_transaction}
    WHERE created >= %d AND allocation = %d AND workflow = %d', $beg_month,
    EC_ALLOC_COMPLETED, EC_WORKFLOW_COMPLETED));
  $mon_sales   = format_currency(db_result(db_query('SELECT SUM(gross)
    FROM {ec_transaction} where created >= %d AND allocation = %d AND workflow = %d',
    $beg_month, EC_ALLOC_COMPLETED, EC_WORKFLOW_COMPLETED)));
  $year_sales  = format_currency(db_result(db_query('SELECT SUM(gross)
    FROM {ec_transaction} where created >= %d AND allocation = %d AND workflow = %d',
    $beg_year, EC_ALLOC_COMPLETED, EC_WORKFLOW_COMPLETED)));

  // Build the content table
  $content = theme('table', NULL, array(
    array(
      array('data' => l(t('Monthly orders'), 'admin/store/transaction',
        array(), 'date=>'. date('m/d/Y', $beg_month)), 'align' => 'right'),
      array('data' => $num_orders, 'align' => 'right')
    ),
    array(
      array('data' => t('Monthly sales'), 'align' => 'right'),
      array('data' => $mon_sales, 'align' => 'right'),
    ),
    array(
      array('data' => t('Yearly sales'), 'align' => 'right'),
      array('data' => $year_sales, 'align' => 'right'),
    ),
  ));

  return $content;
}

/**
 * Called by theme_store_invoice()
 */
function _theme_store_invoice($txn, $print_mode = true, $trial = false) {
  global $base_url;

  $header = array();
  $row    = array();

  if (empty($txn->mail) && $txn->uid > 0) {
    $txn->mail = db_result(db_query('SELECT mail FROM {users} WHERE uid = %d', $txn->uid));
  }

  if (!empty($txn->items)) {
    $header = array(t('Quantity'), t('Item'), t('Price'));

    $shippable = false;
    foreach ($txn->items as $p) {
      $prod = product_load($p);
      if (product_is_shippable($p->vid)) {
        $shippable = true;
      }
      $price = store_adjust_misc($txn, $p);

      $subtotal += (product_has_quantity($p) ? $p->qty * $price : $price);
      $details = '';
      if (0 && is_array($p->data)) {
        foreach ($p->data as $key => $value) {
          if (!empty($value)) {
            $items[] = '<strong>'. check_plain($key) .': </strong>'. check_plain($value);
          }
        }
        if (!empty($items)) {
          $details = theme('item_list', $items);
        }
      }

      $row[] = array(array('data' => $p->qty, 'align' => 'center', 'valign' => 'top'),
        '<em>'. check_plain($p->title) .'</em> '. (($prod->sku != '') ?
        '['. check_plain($prod->sku) .']' : '') .'<br />'. $details,
        array('data' => format_currency($price), 'valign' => 'top', 'align' => 'right'));
    }

    if (is_array($txn->misc)) {
      foreach ($txn->misc as $misc) {
        if (empty($misc->seen)) {
          $price = $misc->qty ? $misc->price * $misc->qty : $misc->price;
          $row[] = array(array('data' => "<strong>{$misc->description}</strong>: ".
            format_currency($price), 'colspan' => 3, 'align' => 'right'));
        }
      }
    }

    $row[] = array(array('data' => '<hr size="1" noshade="noshade" />', 'colspan' => 3, 'align' => 'right'));
    $row[] = array(array('data' => '<strong>'. t('Total:') .'</strong> '.
      format_currency(store_transaction_calc_gross($txn)),
      'colspan' => 3, 'align' => 'right'));
  }

  $payment_info  = '<div><strong>'. t('Ordered On:') .'</strong> '.
    format_date($txn->created) .'</div>';
  if (!empty($txn->duedate)) {
    $payment_info .= '<div><strong>'. t('Due Date:') .'</strong> '.
      format_date($txn->duedate) .'</div>';
  }
  $payment_info .= '<div><strong>'. t('Transaction ID:') .'</strong> '.
    ($trial ? t('Trial Invoice - Not Yet Posted') : $txn->txnid) .'</div>';

  $css        = base_path() . drupal_get_path('module', 'store') .'/invoice.css';
  $site_name  = t('%site-name Invoice', array('%site-name' => variable_get('site_name', 'drupal')));

  include_once drupal_get_path('module', 'store') .'/store.inc';
  if ($shipping_to = theme('store_format_address', $txn, 'shipping', 'html')) {
    $shipping_label = t('Shipping to');
  }
  if ($billing_to = theme('store_format_address', $txn, 'billing', 'html')) {
    $billing_label = t('Billing to');
  }

  if (!empty($txn->ship)) {
    $shipping_method_label = t('Shipping method:');
    $shipping_method = store_format_shipping_method($txn);
  }
  $email_label = t('E-mail:');
  $items_label = t('Items ordered');
  $items_view = theme('table', $header, $row, array('cellpadding' => 3, 'cellspacing' => 3));

  $payment_label = t('Payment Info');

  if (!empty($print_mode)) {
    $output .= <<<EOD
<html>
  <head>
    <style type="text/css" media="all">@import url('$css');</style>
  </head>
  <body>
EOD;
  }

$output .= <<<EOD
    <h1>$site_name</h1>

    <table cellspacing="5">
      <tr>
        <th align="left">$shipping_label</th>
        <th align="left">$billing_label</th>
      </tr>
      <tr>
        <td>$shipping_to</td>
        <td>$billing_to</td>
      </tr>
    </table>

    <p><strong>$shipping_method_label</strong> $shipping_method</p>
    <p><strong>$email_label</strong> $txn->mail</p>

    <h2>$items_label</h2>
    $items_view

    <h2>$payment_label</h2>
    $payment_info
EOD;

if (!empty($print_mode)) {
  $output .= <<<EOD
    </body>
  </html>
EOD;
}

  if (empty($print_mode)) {
    return $output;
  }
  echo $output;
}


